from textwrap import indent

import pytest

from acquire.acquire import MODULES


@pytest.mark.parametrize("module", MODULES.keys())
def test_validate_module_spec(module):
    data_in_spec = []
    for spec in MODULES[module].SPEC:
        type, collectable, *_ = spec
        if type == "glob":
            data_in_spec.append(spec + ("*" in collectable,))
        else:
            data_in_spec.append(spec + ("*" not in collectable,))

    faulty_specs = list(filter(lambda x: x[-1] is False, data_in_spec))
    formatted_specs = "\n".join([f"({spec[0]!r}, {spec[1]!r}) was faulty" for spec in faulty_specs])
    assert len(faulty_specs) == 0, f"{module}:\n{indent(formatted_specs, prefix='    ')}"



from dissect.target import Target
from dissect.target.filesystem import VirtualFilesystem

from acquire.acquire import from_user_home, misc_osx_user_homes, misc_unix_user_homes


def test_from_user_home_osx(mock_target: Target, mock_fs: VirtualFilesystem):
    mock_target.os = "osx"
    expected_results = []
    for user in ["Foo", "Bar"]:
        mock_fs.makedirs(f"/Users/{user}")
        mock_fs.map_file_entry(f"/Users/{user}/application", None)
        expected_results.append(f"/Users/{user}/application")

    assert list(from_user_home(mock_target, "application")) == expected_results


def test_misc_osx_from_user_home(mock_target: Target, mock_fs: VirtualFilesystem):
    mock_fs.makedirs("/root")
    expected_results = ["/root"]
    for user in ["Foo", "Bar"]:
        mock_fs.makedirs(f"/home/{user}")
        expected_results.append(f"/home/{user}")

    assert list(str(home) for home in misc_osx_user_homes(mock_target)) == expected_results


def test_misc_unix_user_homes(mock_target: Target, mock_fs: VirtualFilesystem):
    mock_fs.makedirs("/root")
    expected_results = ["/root"]
    for user in ["Foo", "Bar"]:
        mock_fs.makedirs(f"/home/{user}")
        expected_results.append(f"/home/{user}")

    assert list(str(home) for home in misc_unix_user_homes(mock_target)) == expected_results
